--
-- Manfredas Zabarauskas
-- University of Edinburgh
-- BSc Artificial Intelligence & Computer Science '08
--
-- Informatics 1A Competition Entry
--
-- Text.hs
--

module Text (
              stringToText -- String -> Color -> (Coord, Coord) -> [RasterPoint]
            ) where

import Char

import Render

type Led = (Char, String)

alpha = ['a'..'z']++['0'..'9']++[' ', '/']
alpha_led = zip alpha led_map

stringToText :: String ->          -- input string
                Color  ->          -- text color
                (Coord, Coord) ->  -- position
                [RasterPoint]      -- rastered "led" string
stringToText xs clr (x, y) = (concat . map (\(c, i) -> toLED c clr (x + (spacing 2 * i), y))) $ zip (filter in_alpha (map toLower xs)) [0..]
    where spacing = (length ((led!!0)!!0) +)
          in_alpha x = (not . null . (filter (== x))) alpha

renderLED :: [String] ->    -- "LED"
             Color ->       -- "LED" color
             [RasterPoint]  -- rendered "LED"
renderLED led clr = (concat . (map rasterLine)) (zip led [len_led - 1, len_led - 2..0])
    where 
      len_led = length led
      
      rasterLine :: (String, Int) -> [RasterPoint]
      rasterLine (line, y) = (concat . (map (\(c, x) -> if (c == 'X') then [((x, y), clr)] else []))) (zip line [0..])


toLED :: Char ->             -- input character
         Color ->            -- "led" color
         (Coord, Coord) ->   -- translation
         [RasterPoint]       -- resulting raster points
toLED c clr (t_x, t_y) = map (\((x, y), c) -> ((x + t_x, y + t_y), c)) $ (concat . (map (\(x, i) -> if (x == '1') then renderLED (led!!i) clr else []))) (zip led_index [0..])
    where 
      the [x] = x
      led_index = (snd . the . (filter (\(c', led_m) -> c == c'))) alpha_led


------- LED PIXEL MAPS ------

led_map = ["11100000111100000100",    -- a
           "11100000111100000111",    -- b
           "11100000000100000011",
           "10100010000100001010",
           "11100000011100000011",
           "11100000010100000000",
           "11100000001100000111",
           "00100000111100000100",
           "00000100000000100000",
           "11000000100100000111",
           "00100001010100010000",
           "00100000000100000011",
           "00110001100100000100",    -- .
           "00110000100100010100",    -- .
           "11100000100100000111",    -- .
           "11100000111100000000",
           "11100000100100010111",
           "11100000111100010000",
           "11100000011000000111",
           "11000100000000100000",
           "00100000100100000111",
           "00100000100010001000",
           "00100000100101010100",
           "00010001000001010000",
           "00010001000000100000",
           "11000001000001000011",    -- z
           "11110000100100010111",    -- 0
           "00001100000000100000",
           "11000000111100000011",
           "11000000111000000111",    -- .
           "00100000111000000100",    -- .
           "11100000011000000111",    -- .
           "11100000011100000111",
           "11000000100000000100",
           "11110000111100010111",
           "11100000111000000111",    -- 9
           "00000000000000000000",    -- space
           "00000001000001000000"]    -- forward slash

led    = [[".XXXXXX........",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "..............."],

          ["........XXXXXX.",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "..............."],  

          ["...............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "..............."], 

          ["...............",
           ".X.............",
           ".X.............",
           "..X............",
           "..X............",
           "...X...........",
           "...X...........",
           "....X..........",
           "....X..........",
           ".....X.........",
           ".....X.........",
           "......X........",
           "......X........",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "..............."], 

          ["...............",
           "......X........",
           "......X........",
           ".....X.........",
           ".....X.........",
           "....X..........",
           "....X..........",
           "...X...........",
           "...X...........",
           "..X............",
           "..X............",
           ".X.............",
           ".X.............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "..............."], 

          ["...............",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "..............."], 

          ["...............",
           "........X......",
           "........X......",
           ".........X.....",
           ".........X.....",
           "..........X....",
           "..........X....",
           "...........X...",
           "...........X...",
           "............X..",
           "............X..",
           ".............X.",
           ".............X.",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "..............."], 

          ["...............",
           ".............X.",
           ".............X.",
           "............X..",
           "............X..",
           "...........X...",
           "...........X...",
           "..........X....",
           "..........X....",
           ".........X.....",
           ".........X.....",
           "........X......",
           "........X......",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "..............."], 

          ["...............",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "..............."], 

          ["...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           ".XXXXXX........",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "..............."],

          ["...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "........XXXXXX.",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "..............."],

          ["...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "X..............",
           "..............."],

          ["...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           ".X.............",
           ".X.............",
           "..X............",
           "..X............",
           "...X...........",
           "...X...........",
           "....X..........",
           "....X..........",
           ".....X.........",
           ".....X.........",
           "......X........",
           "......X........",
           "..............."],

          ["...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "......X........",
           "......X........",
           ".....X.........",
           ".....X.........",
           "....X..........",
           "....X..........",
           "...X...........",
           "...X...........",
           "..X............",
           "..X............",
           ".X.............",
           ".X.............",
           "..............."],

          ["...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X.......",
           ".......X......."],

          ["...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "........X......",
           "........X......",
           ".........X.....",
           ".........X.....",
           "..........X....",
           "..........X....",
           "...........X...",
           "...........X...",
           "............X..",
           "............X..",
           ".............X.",
           ".............X.",
           "..............."],

          ["...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           ".............X.",
           ".............X.",
           "............X..",
           "............X..",
           "...........X...",
           "...........X...",
           "..........X....",
           "..........X....",
           ".........X.....",
           ".........X.....",
           "........X......",
           "........X......",
           "..............."],

          ["...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X",
           "..............X"],

          ["...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           ".XXXXXX........"],

          ["...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "........XXXXXX."],

          ["...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "...............",
           "..............."]]


led_f =  [ ".XXXXXX.XXXXXX.",
           "XX....XXX....XX",
           "XX....XXX....XX",
           "X.X..X.X.X..X.X",
           "X.X..X.X.X..X.X",
           "X..XX..X..XX..X",
           "X..XX..X..XX..X",
           "X..XX..X..XX..X",
           "X..XX..X..XX..X",
           "X.X..X.X.X..X.X",
           "X.X..X.X.X..X.X",
           "XX....XXX....XX",
           "XX....XXX....XX",
           ".XXXXXX.XXXXXX.",
           "XX....XXX....XX",
           "XX....XXX....XX",
           "X.X..X.X.X..X.X",
           "X.X..X.X.X..X.X",
           "X..XX..X..XX..X",
           "X..XX..X..XX..X",
           "X..XX..X..XX..X",
           "X..XX..X..XX..X",
           "X.X..X.X.X..X.X",
           "X.X..X.X.X..X.X",
           "XX....XXX....XX",
           "XX....XXX....XX",
           ".XXXXXX.XXXXXX." ]